/**
 *  Copyright (c) 2008...2012 Ekkehard Gentz (Rosenheim, Germany) and others.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors:
 *          Ekkehard Gentz     - initial contribution 
 *     
 *   more info: see folder [project]/copyright
 */
package org.appbus.bb.app.hellologging4bb.logging4bb;

import java.util.Hashtable;
import java.util.NoSuchElementException;
import java.util.Vector;

import net.rim.device.api.system.ApplicationDescriptor;
import net.rim.device.api.system.ApplicationManager;
import net.rim.device.api.system.ApplicationManagerException;
import net.rim.device.api.system.CodeModuleManager;

/**
 * The Factory gives you an ILogger for each Class
 * it's recommended in each Class to define
 * private static final ILogger logger = LoggerFactory.getLogger(MyClass.class);
 * then you can log as usual logger.info("bla") etc
 * 
 * @author ekkehard gentz, rosenheim, germany
 *
 */
public class LoggerFactory {
	// in this special case the ILogger is not final an will be set while initializing
	private static ILogger logger;
	
	private static boolean logger4bbIsAvailable = false;
	private static final int APP_DOESNT_EXIST = 0;
	private static boolean isInitialized = false;
	private static Vector events = new Vector(500);
	private static boolean isWorking = false;
	private static boolean haveMoreWorkToDo = false;
	private static WorkerThread theThread;
	
	////////////////////////////////////////////////////////////////////////////
	//////////   A T T E N T I O N                     /////////////////////////
	//////////   check if some values have to changed  /////////////////////////
	// TODO must match MY_APPLICATION_CONTEXT_CATEGORY in Logging4BBApp class
	// if testing against product from AppWorld you have to use: "ekkes_logging4bb"
	public static String LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY = "logging4bb";
	// TODO change the name to be used as your own Appname displayed in DB and Lilith
	private static String MY_APP_NAME = "HelloLogging";
	// ** optional:** 
	// TODO if using a STORE_ID this must be a unique ID
	// TODO org.appbus.bb.app.hellologging4bb.logging4bb.LogStoreID
	// TODO 0xb5d14cb9be19b553L
	public static final Long MY_OWN_STORE_ID = new Long(0xb5d14cb9be19b553L);
	// TODO Worker sends all queued events + tries again after xxx ms
	private static final int DELAY_NEXT_REMOTE_COMMAND_EXECUTION_MS = 500;
	// TODO if nothing is queued, then the Worker sleeps for xxx ms
	private static final int DELAY_SLEEP_BEFORE_TEST_FOR_NEXT_EVENTS_MS = 2000;
	////////////////////////////////////////////////////////////////////////////
	
	/**
	 * initializes the Logger:
	 * RemoteLogger if logging4bb is running
	 * FallbackLogger if logging4bb is not running
	 */
	private synchronized static void initLogger4bb(String appName, int defaultLogLevel){
		if (isInitialized) {
			return;
		}
		// test if logger4bb is running and remote commands available
		if (isLogging4BBApplicationInstalled()) {
			System.out.println("init Remote Logger");
			logger4bbIsAvailable = true;
			startupWorker();
			RemoteLogger.init(appName, defaultLogLevel);
			logger = new RemoteLogger(LoggerFactory.class);
			logger.debug("RemoteLogger for LoggerFactory created");
		} else {
			System.out.println("init Fallback Logger");
			logger4bbIsAvailable = false;
			FallbackLogger.init(appName, defaultLogLevel);
			logger = new FallbackLogger(LoggerFactory.class);
			logger.debug("FallbackLogger for LoggerFactory created");
		}
		isInitialized = true;
	}

	/**
	 * provides you with a ILogger specific for Class clazz
	 * 
	 * @param clazz
	 * @return
	 */
	public static ILogger getLogger(Class clazz){
		System.out.println("want to get Logger for Class: "+clazz);
		if (!isInitialized) {
			initLogger4bb(MY_APP_NAME, ILogger.LOG_LEVEL_DEBUG);
		}
		if (logger4bbIsAvailable) {
			logger.debug("get RemoteLogger for "+clazz.getName());
			return new RemoteLogger(clazz);
		} else {
			logger.debug("get FallbackLogger for "+clazz.getName());
			return new FallbackLogger(clazz);
		}
	}
	
	/**
	 * checks if the background app is installed and running
	 * if installed but not running, the app is started
	 * 
	 * @return
	 */
	public static boolean isLogging4BBApplicationInstalled(){
		int handle = CodeModuleManager.getModuleHandle(LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY);
		if (handle == APP_DOESNT_EXIST) {
			System.out.println("no remote app installed: "+LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY);
			return false;
		}
		System.out.println("remote logging app is installed :)");
		ApplicationManager manager = ApplicationManager.getApplicationManager();
		ApplicationDescriptor[] applicationDescriptors = manager.getVisibleApplications();
		for (int i = 0; i < applicationDescriptors.length; i++) {
			ApplicationDescriptor applicationDescriptor = applicationDescriptors[i];
			if (applicationDescriptor.getModuleName().equals(LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY)) {
				System.out.println("app is running");
				return true;
			}
		}
		applicationDescriptors =
			CodeModuleManager.getApplicationDescriptors(handle);
		if (applicationDescriptors.length == 0) {
			System.out.println("no ApplicationDescriptor found for Code handle");
			return false;
		}
		try {
			manager.runApplication(applicationDescriptors[0], false);
			return true;
		} catch (ApplicationManagerException e) {
			System.out.println("cannot run the background app");
		}
		return false;
	}
	
	/**
	 * adds one LogEvent to the buffer
	 * events will be sent to remote app from async worker thread
	 * 
	 */
	public static void addEvent(Hashtable event){
		synchronized (events) {
			events.addElement(event);
		}
	}
	
	/**
	 * tries to get the next event 
	 * if there's no next event NoSuchElementException was thrown
	 * so the calling Thread can go to sleep mode
	 * 
	 * @return
	 * @throws NoSuchElementException
	 */
	public static Hashtable[] getNextEvents() throws NoSuchElementException{
		synchronized (events) {
			if (events.isEmpty()) {
				throw new NoSuchElementException("Worker queue is empty");
			}
			Hashtable[] events2send = new Hashtable[events.size()];
			events.copyInto(events2send);
			events.removeAllElements();
			return events2send;
		}
	}
	
	/**
	 * Worker Thread must only be started if Remote Logger is available
	 */
	public static void startupWorker(){
		isWorking = true;
		theThread = new WorkerThread();
		theThread.start();
	}
	
	/**
	 * should be called if your App is closed
	 * to stop all running threads etc.
	 */
	public static void shutdownWorker(){
		if (theThread != null) {
			theThread.interrupt();
			theThread = null;
		}
	}
	
	/**
	 * Worker Thread sends always a bunch of events to the remote app
	 * if many events happen and all are send as single event thru Remote Commands
	 * it could happen that the Background App isn't responding anymore
	 * 
	 * so the Thread always gets all queued events from the events Vector
	 * and sends all events to the background app logging4bb
	 * then waits DELAY_NEXT_REMOTE_COMMAND_EXECUTION_MS ms before sending the next
	 * if the Queue is empty thje worker waits DELAY_SLEEP_BEFORE_TEST_FOR_NEXT_EVENTS_MS ms
	 * before testing again if there are queued events
	 *
	 */
	public static class WorkerThread extends Thread {
		public void run() {
			try {
				while (isWorking || haveMoreWorkToDo) {
					try {
						RemoteLoggerCommand.logEventCommandRemoteExecute(getNextEvents());
						sleep(DELAY_NEXT_REMOTE_COMMAND_EXECUTION_MS);
					} catch (NoSuchElementException e) {
						if (isWorking) {	
							// nothing to do - we sleep and try again
							sleep(DELAY_SLEEP_BEFORE_TEST_FOR_NEXT_EVENTS_MS);
						} else {
							haveMoreWorkToDo = false;
						}
					}
				}
			} catch (InterruptedException e) {
				// if interrupted all queued work will be processed
				// if you want to kill al, set haveMoreWorkToDo also to false
				isWorking = false;
			}
		}
	}
}
