/**
 *  Copyright (c) 2008...2012 Ekkehard Gentz (Rosenheim, Germany) and others.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors:
 *          Ekkehard Gentz     - initial contribution 
 *     
 *   more info: see folder [project]/copyright
 */
package org.appbus.bb.app.hellologging4bb.logging4bb;

import java.util.Hashtable;

import net.rim.device.api.system.Characters;
import net.rim.device.api.system.DeviceInfo;

/**
 * Logger implements ILogger
 * and logs to logging4bb app via RemoteCommands
 * 
 * LogEvents are encapsulated in a Hashtable
 * 
 * @author ekke
 *
 */
public class RemoteLogger implements ILogger {
	
	private static int defaultLogLevel = LOG_LEVEL_INFO;
	private static String appName4Socket;
	private static String appName4DB;
	
	// its a good idea to have a separate Logger for each Class
	private Class clazz;
	
	public RemoteLogger(Class clazz) {
		this.clazz = clazz;
	}
	
	public static boolean init(
			String appName, 
			int defaultLogLevel) {
		RemoteLogger.appName4DB = constructAppNameForDB(appName);
		RemoteLogger.appName4Socket = constructAppNameForSocket(appName);
		RemoteLogger.defaultLogLevel = defaultLogLevel;
    	System.out.println("RemoteLogger initialized for App: "+appName);
        return true;
    }
	
	/**
	 * adds the Device and PIN to the AppName
	 * 
	 * @param appName
	 * @return
	 */
	private static String constructAppNameForSocket(String appName){
		StringBuffer sb = new StringBuffer();
		sb.append(appName);
		sb.append(Characters.LEFT_CURLY_BRACKET);
		sb.append(DeviceInfo.getDeviceName());
		sb.append(Characters.COLON);
		sb.append(Integer.toHexString(DeviceInfo.getDeviceId()));
		sb.append(Characters.RIGHT_CURLY_BRACKET);
		return sb.toString();
	}
	
	/**
	 * adds the Device to the AppName
	 * 
	 * @param appName
	 * @return
	 */
	private static String constructAppNameForDB(String appName){
		StringBuffer sb = new StringBuffer();
		sb.append(appName);
		sb.append(Characters.LEFT_CURLY_BRACKET);
		sb.append(DeviceInfo.getDeviceName());
		sb.append(Characters.RIGHT_CURLY_BRACKET);
		return sb.toString();
	}
	
	/**
	 * logs one event:
	 * transforms the Event data into a Hashtable
	 * (Hashtables with String, Long, Integer as Types can easy be sent to the background app)
	 * 
	 * events are not sent immediately - they are buffered
	 * and an async Task is waiting to send a bunch of events
	 * to the background logger app
	 * 
	 * if sending each event separately it could happen that it's too fast
	 * for the RemoteCommands registry and background app is not responding anymore
	 * 
	 * @param context
	 * @param message
	 * @param level
	 * @param exception
	 */
	private void logEvent(String context, String message, int level, Throwable exception){
		if (defaultLogLevel > level) {
			return;
		}
		Hashtable remoteEvent = new Hashtable(12);
		remoteEvent.put(ILogger.APPNAMEDB_KEY, appName4DB);
		remoteEvent.put(ILogger.APPNAMESOCKET_KEY, appName4Socket);
		if (context != null && context.length() > 0) {
			remoteEvent.put(ILogger.CONTEXT_KEY, context);
		}
		remoteEvent.put(ILogger.MESSAGE_KEY, message);
		remoteEvent.put(ILogger.EVENTTIME_KEY, new Long(System.currentTimeMillis()));
		if (exception != null) {
			remoteEvent.put(ILogger.EXCEPTION_KEY, exception);
		}
		remoteEvent.put(ILogger.LEVEL_KEY, new Integer(level));
		remoteEvent.put(ILogger.LOGGERNAME_KEY, clazz.getName());
		
		// optional
		remoteEvent.put(ILogger.LOGSTOREID_KEY, LoggerFactory.MY_OWN_STORE_ID);
		
		// sending of events to remote app will be asynchron
		LoggerFactory.addEvent(remoteEvent);
	}

	public void debug(String logMessage) {
		logEvent(null, logMessage, LOG_LEVEL_DEBUG, null);
	}
	
	public void debug(String logMessage, Throwable exception) {
		logEvent(null, logMessage, LOG_LEVEL_DEBUG, exception);
	}

	public void debug(String context, String logMessage) {
		logEvent(context, logMessage, LOG_LEVEL_DEBUG, null);
	}

	public void debug(String context, String logMessage, Throwable exception) {
		logEvent(context, logMessage, LOG_LEVEL_DEBUG, exception);
	}
	
	public void info(String logMessage) {
		logEvent(null, logMessage, LOG_LEVEL_DEBUG, null);
	}
	
	public void info(String logMessage, Throwable exception) {
		logEvent(null, logMessage, LOG_LEVEL_DEBUG, exception);
	}

	public void info(String context, String logMessage) {
		logEvent(context, logMessage, LOG_LEVEL_DEBUG, null);
	}

	public void info(String context, String logMessage, Throwable exception) {
		logEvent(context, logMessage, LOG_LEVEL_DEBUG, exception);
	}
	
	public void log(String logMessage) {
		logEvent(null, logMessage, LOG_LEVEL_INFO, null);
	}
	
	public void log(String logMessage, Throwable exception) {
		logEvent(null, logMessage, LOG_LEVEL_INFO, exception);
	}

	public void log(String context, String logMessage) {
		logEvent(context, logMessage, LOG_LEVEL_INFO, null);
	}

	public void log(String context, String logMessage, Throwable exception) {
		logEvent(context, logMessage, LOG_LEVEL_INFO, exception);
	}
	
	public void warn(String logMessage) {
		logEvent(null, logMessage, LOG_LEVEL_WARN, null);
	}
	
	public void warn(String logMessage, Throwable exception) {
		logEvent(null, logMessage, LOG_LEVEL_WARN, exception);
	}

	public void warn(String context, String logMessage) {
		logEvent(context, logMessage, LOG_LEVEL_WARN, null);
	}

	public void warn(String context, String logMessage, Throwable exception) {
		logEvent(context, logMessage, LOG_LEVEL_WARN, exception);
	}

	public void error(String logMessage) {
		logEvent(null, logMessage, LOG_LEVEL_ERROR, null);
	}
	
	public void error(String logMessage, Throwable exception) {
		logEvent(null, logMessage, LOG_LEVEL_ERROR, exception);
	}

	public void error(String context, String logMessage) {
		logEvent(context, logMessage, LOG_LEVEL_ERROR, null);
	}

	public void error(String context, String logMessage, Throwable exception) {
		logEvent(context, logMessage, LOG_LEVEL_ERROR, exception);
	}

	/**
	 * test if debug is enabled
	 */
	public boolean isDebugEnabled() {
		return defaultLogLevel == LOG_LEVEL_DEBUG;
	}

}
