/**
 *  Copyright (c) 2008...2012 Ekkehard Gentz (Rosenheim, Germany) and others.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors:
 *          Ekkehard Gentz     - initial contribution 
 *     
 *   more info: see folder [project]/copyright
 */
package org.appbus.bb.app.hellologging4bb.logging4bb;

import java.util.Hashtable;

import net.rim.device.api.command.Command;
import net.rim.device.api.command.registrar.CategoryCollection;
import net.rim.device.api.command.registrar.CommandRequest;
import net.rim.device.api.command.registrar.RemoteCommandRegistrarConnection;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.Dialog;

/**
 * LogEvents are send to the background app via RemoteCommand
 * its also possible to restart the Logger
 * perhaps if Lilith was down or SD Card was unmounted etc.
 * 
 * @author ekke
 *
 */
public class RemoteLoggerCommand {
	
	private static final ILogger logger = LoggerFactory.getLogger(RemoteLoggerCommand.class);
	
	// only used for workaround BB OS 6 bug not closing remote command registry entries
	private static String COMMAND_ID_DELIMITER = "@";

	// helper to notify the user if there are problems
	private static void notifyUI(final String message){
		UiApplication.getUiApplication().invokeLater(
		new Runnable()		{
				public void run() {
					Dialog.inform(message);
				};
			}
		);
	}
	
	private static String[] logEventCommandCategories={"logEventFromRemoteClient"};
	private static String[] logEventContextCategories={LoggerFactory.LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY};
	/**
	 * logs one or more log events to the remote app
	 * using a Hastable[], where each Hashtable conatins one log event
	 * 
	 * @param events
	 * @return
	 */
	public static boolean logEventCommandRemoteExecute(Hashtable[] events){
		CommandRequest request = new CommandRequest();
    	request.setCommandCategories(new CategoryCollection(logEventCommandCategories));
    	request.setContextCategories(new CategoryCollection(logEventContextCategories));
    	request.setAndContextCategories(true);
    	RemoteCommandRegistrarConnection connection = new RemoteCommandRegistrarConnection();
    	Command[] logEventCmds = connection.getCommands(request);
    	Command logEventCmd = getNewestCommand(logEventCmds);
    	if (logEventCmd == null) {
    		// hint: if the logger wasn't running and your app
    		// starts the app - perhaps remote commands are not registered yet
    		// then you have to delay the first execution
    		// but from my experiences it never happens in reality
    		// and IF it happens: only some log events are lost
			notifyUI("No Remote Command found - is Background App logging4bb running ?");
			return false;
		}
    	logEventCmd.execute(events);
    	return true;
	}
	
	private static String[] resumeLoggerCommandCategories={"resumeLogger"};
	private static String[] resumeLoggerContextCategories={LoggerFactory.LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY};
	/**
	 * restarts the Logger
	 * if the logger was paused before you can restart the logger again
	 * 
	 * or if Lilith wasn't available at last startup
	 * a restart (resume) will create a new Connection
	 * 
	 * @return
	 */
	public static boolean resumeLoggerCommandRemoteExecute(){
		CommandRequest request = new CommandRequest();
    	request.setCommandCategories(new CategoryCollection(resumeLoggerCommandCategories));
    	request.setContextCategories(new CategoryCollection(resumeLoggerContextCategories));
    	request.setAndContextCategories(true);
    	RemoteCommandRegistrarConnection connection = new RemoteCommandRegistrarConnection();
    	Command[] resumeLoggerCmds = connection.getCommands(request);
    	Command resumeLoggerCmd = getNewestCommand(resumeLoggerCmds);
    	if (resumeLoggerCmd == null) {
			notifyUI("No Remote Command found - is Background App logging4bb running ?");
			return false;
		}
    	resumeLoggerCmd.execute(new Hashtable());
    	return true;
	}	
	
	private static String[] pauseLoggerCommandCategories={"pauseLogger"};
	private static String[] pauseLoggerContextCategories={LoggerFactory.LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY};
	/**
	 * restarts the Logger
	 * if the logger was paused before you can restart the logger again
	 * 
	 * or if Lilith wasn't available at last startup
	 * a restart (resume) will create a new Connection
	 * 
	 * @return
	 */
	public static boolean pauseLoggerCommandRemoteExecute(){
		CommandRequest request = new CommandRequest();
    	request.setCommandCategories(new CategoryCollection(pauseLoggerCommandCategories));
    	request.setContextCategories(new CategoryCollection(pauseLoggerContextCategories));
    	request.setAndContextCategories(true);
    	RemoteCommandRegistrarConnection connection = new RemoteCommandRegistrarConnection();
    	Command[] pauseLoggerCmds = connection.getCommands(request);
    	Command pauseLoggerCmd = getNewestCommand(pauseLoggerCmds);
    	if (pauseLoggerCmd == null) {
			notifyUI("No Remote Command found - is Background App logging4bb running ?");
			return false;
		}
    	pauseLoggerCmd.execute(new Hashtable());
    	return true;
	}	
	
	// these columns can be changed:
    public static String COL_HOST_ADDRESS = "HOST_ADDRESS";
    public static String COL_LOG_TO_DB = "LOG_TO_DB";
    public static String COL_LOG_TO_LOGMONITOR = "LOG_TO_LOGMONITOR";
    
	private static String[] refreshLoggerCommandCategories={"refreshPreferencesFromDatabase"};
	private static String[] refreshLoggerContextCategories={LoggerFactory.LOGGING4BB_APP_NAME_AND_CONTEXT_CATEGORY};
	/**
	 * refreshes the Logger
	 * 
	 * you can change some preferences and refresh logger porpertzies of remote app
	 * 
	 * @return
	 */
	public static boolean refreshLoggerCommandRemoteExecute(Hashtable columns){
		CommandRequest request = new CommandRequest();
    	request.setCommandCategories(new CategoryCollection(refreshLoggerCommandCategories));
    	request.setContextCategories(new CategoryCollection(refreshLoggerContextCategories));
    	request.setAndContextCategories(true);
    	RemoteCommandRegistrarConnection connection = new RemoteCommandRegistrarConnection();
    	Command[] refreshLoggerCmds = connection.getCommands(request);
    	Command refreshLoggerCmd = getNewestCommand(refreshLoggerCmds);
    	if (refreshLoggerCmd == null) {
			notifyUI("No Remote Command found - is Background App logging4bb running ?");
			return false;
		}
    	refreshLoggerCmd.execute(columns);
    	return true;
	}	
	
	// Helper for workaround bugfix BB OS 6 not removing entries from remoteRegistrar
	private static Command getNewestCommand(Command[] commands){
		if (commands == null || commands.length == 0) {
			return null;
		}
		if (commands.length == 1) {
			return commands[0];
		}
		long highScoreTimestamp = Long.MIN_VALUE;
		int indexOfHighScoreCommand = 0;
		for (int i = 0; i < commands.length; i++) {
			String commandId = commands[i].getId();
			int posOfDelimiter = commandId.indexOf(COMMAND_ID_DELIMITER);
			if (posOfDelimiter > 0) {
				try {
					String timestampAsString = commandId.substring(posOfDelimiter+1);
					long timestamp = Long.parseLong(timestampAsString);
					if (timestamp > highScoreTimestamp) {
						highScoreTimestamp = timestamp;
						indexOfHighScoreCommand = i;
					}
				} catch (IndexOutOfBoundsException e) {
					// ignore
				} catch (NumberFormatException e) {
					// ignore
				}
			} 
		}
		// should not happen because logging4bb is started as a system module
		// but if you change this and allow logging4bb to be started / stopped
		// then the OS 6 bug will happen because registered remote commands
		// are not closed at program exit
		logger.debug("OS 6 Workaround", "found commands: "+commands.length);
		return commands[indexOfHighScoreCommand];
	}

}
